'use strict'

let Path = require('path');
let Iconv = require('iconv-lite');
let Fs = require('fs');
let Url = require('url');


module.exports = {
	root: 'mockhs',
	info: function(msg) {
		console.info('[info] ' + msg);
	},
	error: function(msg) {
		console.error('[error] ' + msg);
	},
	log: function(msg) {
		console.log('[log] ' + msg);
	},
	debug: function(msg){
		msg = typeof msg == 'object' ? JSON.stringify(msg) : msg;
		console.log('[mockhs] ' + msg);
	},
	undef: function(before, after){
	    return before ? before : after;
	},
	rewrite: function(map, url) {
		for(var i = 0; i < map.length; i++){
			let val = map[i];

			if (val.length != 2) {
				continue;
			}
			let from = val[0];
			let to = val[1];
			let reg = new RegExp('^'+from+'(\\?[^\\?]+)?$', 'g');
			let index = url.indexOf(from);
			let parsed = Url.parse(url);
			let parsedFrom = Url.parse(from);
			let isEqual = parsed.pathname == parsedFrom.pathname;
			//带端口号的请求parsed无host属性
			let isUrl = reg.test(url) || (parsed && isEqual);
            let isInclude = url.indexOf(from) !== -1 && /^http/.test(url);

			//目录匹配
			if(isInclude && !isUrl){

				let before = url.slice(from.length);
				let result = Path.resolve(to+before);
				result = this.toless(result);
				result = result.replace(/\?.*$/g, '');
				return {
					url: result,
					type: 'root'
				};
			}

			//url匹配
			if(isUrl){

				return {
					url: to,
					type: 'url'
				};
			}
		}
		return {
			url: url,
			type: 'url'
		};
	},
	setResponse: function(response, contentType, buffer) {
		response.setHeader('Content-Type', contentType);
		response.setHeader('Access-Control-Allow-Origin', '*');
		response.write(buffer);
		response.end();
	},
	readFileSync: function(filePath, encoding) {
		var buffer = new Buffer('');

		try {
			buffer = Fs.readFileSync(filePath);
		} catch (e) {
			error(e.toString());
		}

		if (!encoding) {
			return buffer;
		}

		var fileStr = Iconv.fromEncoding(buffer, encoding);

		return fileStr;
	},
	removeVersion: function(url) {
		return url.replace(/(?!.*?\_[a-z]+\..*?$)([^?]+)(?:_|\.)[0-9a-z]{7}(\.(?:js|css|jpg|jpeg|png|gif))/i, '$1$2');
	},
	//css to less
	toless: function(url) {
		return url.replace(/\.css$/, '.less');
	}
}
